library graph.view;

/**
 * This package implements the graph component, represented by the mxGraph
 * class. The graph holds an mxGraphModel which contains mxCells and caches the
 * state of the cells in mxGraphView. The cells are painted using a canvas from
 * the canvas package. The style of the graph is represented by the mxStylesheet
 * class.
 */

import 'dart:html' show Element, Node;
import 'dart:math' as Math;
import 'dart:collection' show LinkedList, LinkedHashSet, HashSet;
//import 'dart:svg' as svg;

import '../util/java/math.dart' as math;
import '../util/awt/awt.dart' as awt;
import '../util/property_change/property_change.dart';

import '../layout/layout.dart' show IGraphLayout;

import '../canvas/canvas.dart' show Graphics2DCanvas;
import '../canvas/canvas.dart' show ICanvas;
import '../canvas/canvas.dart' show ImageCanvas;

import '../model/model.dart' show Geometry;
import '../model/model.dart' show IGraphModel;
import '../model/model.dart' show GraphModel;
import '../model/model.dart' show ICell;
import '../model/model.dart' show Cell;
import '../model/model.dart' show ChildChange;
import '../model/model.dart' show CollapseChange;
//import '../model/model.dart' show Filter;
import '../model/model.dart' show GeometryChange;
import '../model/model.dart' show RootChange;
import '../model/model.dart' show StyleChange;
import '../model/model.dart' show TerminalChange;
import '../model/model.dart' show ValueChange;
import '../model/model.dart' show VisibleChange;

//import '../view/view.dart' show EdgeStyleFunction;
//import '../view/view.dart' show PerimeterFunction;

import '../util/util.dart' show Constants;
import '../util/util.dart' show Utils;
import '../util/util.dart' show Point2d;
import '../util/util.dart' show Rect;
import '../util/util.dart' show Event;
import '../util/util.dart' show EventObj;
import '../util/util.dart' show EventSource;
import '../util/util.dart' show IEventListener;
import '../util/util.dart' show UndoableEdit;
import '../util/util.dart' show UndoableChange;
import '../util/util.dart' show ImageBundle;
import '../util/util.dart' show Resources;
import '../util/util.dart' show StyleUtils;

part 'cell_state.dart';
part 'graph.dart';
part 'connection_constraint.dart';
part 'edge_style.dart';
part 'graph_view.dart';
part 'graph_selection_model.dart';
part 'layout_manager.dart';
part 'multiplicity.dart';
part 'perimeter.dart';
part 'space_manager.dart';
part 'style_registry.dart';
part 'stylesheet.dart';
part 'swimlane_manager.dart';
part 'temporary_cell_states.dart';
